#pragma once
#include <memory>

#include "XmlElement.h"

class HtmlElement : public XmlElement
{
public:
    enum class Type
    {
        NONE,
        BODY,
        HEAD,
        PARAGRAPH,
        TEXT_RUN,
        CODE,
        HEADING
    };

    HtmlElement(const std::string& tagName);

    virtual Type getType() const = 0;
};

class HtmlCodeElement : public HtmlElement
{
public:
    HtmlCodeElement() : HtmlElement("code")
    {

    }

    Type getType() const override
    {
        return Type::CODE;
    }
};

class HtmlHeadingElement : public HtmlElement
{
public:
    HtmlHeadingElement(unsigned index) : HtmlElement("h" + std::to_string(index))
    {

    }

    Type getType() const override
    {
        return Type::HEADING;
    }
};

using HtmlElementUPtr = std::unique_ptr<HtmlElement>;
